@@ -327,6 +327,9 @@ void StateNetwork::writeStateNetwork(std::string filename) const
 
 	SafeOutFile outFile(filename);
 
+	outFile << "# v" << INFOMAP_VERSION << "\n" <<
+		"# ./Infomap " << m_config.parsedString << "\n";
+
 	if (!m_names.empty()) {
 		outFile << "*Vertices\n";
 		for (auto& nameIt : m_names) {
@@ -363,6 +366,11 @@ void StateNetwork::writePajekNetwork(std::string filename) const
 
 	SafeOutFile outFile(filename);
 
+	outFile << "# v" << INFOMAP_VERSION << "\n" <<
+		"# ./Infomap " << m_config.parsedString << "\n";
+	if (m_config.isMemoryNetwork())
+		outFile << "# State network as physical network\n";
+
 	outFile << "*Vertices\n";
 	for (const auto& nodeIt : nodes()) {
 		const auto& node = nodeIt.second;
@@ -376,7 +384,7 @@ void StateNetwork::writePajekNetwork(std::string filename) const
 		outFile << "\"\n";
 	}
 
-	outFile << "*Links\n";
+	outFile << (m_config.printAsUndirected() ? "*Edges" : "*Arcs") << "\n";
 	for (auto& linkIt : m_nodeLinkMap) {
 		for (auto& subIt : linkIt.second)
 		{
